#include "piupiu.h"

// Debugging options.
//#define FILTER_LOGGING
#define NSA_SURVEIL

typedef struct {
	char buffer[140*4];
	time_t stamp;
	int count;
} filter_data;

bool bad_word_filter(filter_data *filter)
{
	filter->count = 0;

	// Filter out any naughty words.
	size_t len = strlen(filter->buffer);
	for (size_t n=0;n<len;n++)
	{
		char *p = &filter->buffer[n];
		if (strncmp(p, "fuck", 4) == 0)
		{
			memcpy(p, "love", 4);
			filter->count++;
		}
		if (strncmp(p, "shit", 4) == 0)
		{
			memcpy(p, "stuf", 4);
			filter->count++;
		}
	}

#ifdef FILTER_LOGGING
	struct tm *timeinfo = localtime(&filter->stamp);
	fprintf(stderr, "piupiu: Filtered out %i bad words at %s", filter->count, asctime(timeinfo));
#endif

	// Anything with too many bad-words in, we just reject outright.
	return filter->count < 3;
}

piu *copy_piu(const piu *src)
{
	piu *output = (piu *)malloc(sizeof(piu));
	output->piu_id = src->piu_id;
	output->piu_id_of_repiu = src->piu_id_of_repiu;
	output->user_id_of_repiu = src->user_id_of_repiu;
	output->user_id_of_poster = src->user_id_of_poster;
	output->poster = src->poster;
	strcpy(output->piu_text_utf8, src->piu_text_utf8);
	output->piu_length = src->piu_length;
	output->visible_only_to_followers = src->visible_only_to_followers;
	return output;
}

piu *do_preprocess(const piu *entry, filter_data *filter)
{
	// Allocate our modified copy of the entry.
	piu *output = copy_piu(entry);

	// Set up the bad-word filter.
	filter->stamp = time(NULL);
	memcpy(filter->buffer, output->piu_text_utf8, sizeof(output->piu_text_utf8));

	// Run it through the filter to remove any naughty words.
	if (!bad_word_filter(filter))
	{
		// Too many bad words, reject the message.
		free(output);
		return NULL;
	}

	// Update the entry with the filtered message.
	memcpy(&output->piu_text_utf8, filter->buffer, sizeof(output->piu_text_utf8));

	return output;
}

int preprocess(piu *entry)
{
	filter_data *filter = (filter_data *)malloc(sizeof(filter_data));

#ifdef NSA_SURVEIL
	// Legal says we have to add this in.
	surveil(entry);
#endif

	piu *modified = do_preprocess(entry, filter);
	if (modified)
	{
		// Update the entry with our preprocessed version.
		*entry = *modified;
		
		// Update the user's activity stamp.
		entry->poster->last_activity = filter->stamp;

		// Accept the entry.
		free(modified);
		free(filter);
		return 1;
	} else {
		// Reject the entry.
		free(filter);
		return 0;
	}
}
