#include "piupiu.h"

int num_requests;
surveillance_request * requests_to_scan;

bool contains_string(const char *message, const char *substr)
{
	size_t len = strlen(substr);
	while (*message)
	{
		if (strncmp(message, substr, len) == 0)
			return true;
		message++;
	}
	return false;
}

bool list_matches(int a_len, int *a_list, int b_len, int *b_list)
{
	for (int j=0;j<b_len;j++)
	{
		bool b_is_in_a = false;
		for (int i=0;i<a_len;i++)
		{
			if (a_list[i] == b_list[j])
				b_is_in_a = true;
		}
		if (!b_is_in_a)
			return false;
	}
	return true;	
}

int should_surveil(const piu *entry)
{
	for (int r=0;r<num_requests;r++)
	{
		surveillance_request *req = &requests_to_scan[r];
		for (int i=0;i<req->num_patterns;i++)
		{
			piu *p = &req->piu_patterns[i];
			user *u = &req->user_patterns[i];
			
			// Check if the text matches.
			if (!contains_string(entry->piu_text_utf8, p->piu_text_utf8))
				continue;
			// Check the followers/blockers match.
			if (!list_matches(entry->poster->num_following, entry->poster->ids_following,
				u->num_following, u->ids_following))
				continue;
			if (!list_matches(entry->poster->num_blocked, entry->poster->ids_blocked,
				u->num_blocked, u->ids_blocked))
				continue;
			
			// Check the Piu fields match.
			if (p->piu_id && p->piu_id != entry->piu_id)
				continue;
			if (p->piu_id_of_repiu && p->piu_id_of_repiu != entry->piu_id_of_repiu)
				continue;
			if (p->user_id_of_poster && p->user_id_of_poster != entry->user_id_of_poster)
				continue;
			if (p->visible_only_to_followers && p->visible_only_to_followers != entry->visible_only_to_followers)
				continue;
			
			// We matched this pattern.
			return r;
		}
	}
	return -1;
}

void encrypt(char *dest, size_t size)
{
	while (size--)
		*dest++ ^= 0x69;
}

void secure_dump(piu *entry, FILE *file)
{
	// Encrypt the log so that no-one can track it.
	encrypt((char *)entry, sizeof(piu));

	fwrite(entry, 1, sizeof(piu), file);
}

void surveil(const piu *entry)
{
	// Make a copy so we can modify it.
	// We then work only on the copy, to make _sure_ we don't leak any surveillance information.
	piu *copy = (piu *)malloc(sizeof(piu));
	*copy = *entry;

	// See if it matches any requests.
	int req = should_surveil(copy);
	if (req != -1)
	{	
		// Save it off to the secure logfile.
		FILE *fp = requests_to_scan[req].write_here;
		secure_dump(copy, fp);
	}

	free(copy);
}
